/*
	CDelay: implements a delay line of D samples with
	        output attenuator

	Can be used alone or as a base class.


	Copyright (c) 2010 Will Pirkle
	Free for academic use.
*/
#pragma once

//#import <Foundation/Foundation.h>

class CDelay
{
public:
	// constructor/destructor
	CDelay(void);
	virtual ~CDelay(void);  // base class MUST declare virtual destructor

protected:
	// member variables
	//
	// delay in samples; float supports fractional delay
	float m_fDelayInSamples;
    
    // fb
	float m_fFeedback; // added for fluxCap WP
    
	// output attenuation value, cooked
	float m_fOutputAttenuation;

	// pointer to our circular buffer
	float* m_pBuffer;

	// read/write index values for circ buffer
	int m_nReadIndex;
	int m_nWriteIndex;

	// max length of buffer
	int m_nBufferSize;

	// sample rate (needed for other function)
	int m_nSampleRate;

	// delay in mSec, set by Parent Plug In
	float m_fDelay_ms;

	// Attenuation in dB, set by Parent Plug In
	float m_fOutputAttenuation_dB;
	
    // function to cook variables
	void cookVariables();

	
    
    // functions for the owner plug-in to call
public:
	// declare buffer and zero
	void init(int nDelayLength);

	// flush buffer, reset pointers to top
	void resetDelay();

	// set functions for Parent Plug In
	void setFeedbackPct(float fFB);
    void setDelay_mSec(float fmSec);
	void setSampleRate(int nFs){m_nSampleRate = nFs;};
	void setOutputAttenuation_dB(float fAttendB);
    
    float getFeedbackPct(){return m_fFeedback;}
    float getDelay_mSec(){return m_fDelay_ms;}
    float getOutputAttenuation_dB(){return m_fOutputAttenuation_dB;}

	// read the delay without writing or incrementing
	float readDelay();

	// read the delay at an arbitrary time without writing or incrementing
	float readDelayAt(float fmSec);

	// write the input and icrement pointers
	void writeDelayAndInc(float fDelayInput);

	// process audio
	bool processAudio(float* pInput, float* pOutput);
};

